@@ -51,6 +51,7 @@ struct _DcmFilehandle {
     uint32_t frame_width;
     uint32_t frame_height;
     uint32_t num_frames;
+    uint32_t frame_offset;
     struct PixelDescription desc;
     DcmLayout layout;
 
@@ -83,6 +84,12 @@ struct _DcmFilehandle {
     // push and pop these while we parse
     UT_array *dataset_stack;
     UT_array *sequence_stack;
+
+    // skip to tags during parse
+    uint32_t *skip_to_tags;
+
+    // set if we see an ext offset table
+    bool have_extended_offset_table;
 };
 
 
@@ -225,8 +232,8 @@ static bool dcm_require(DcmError **error, DcmFilehandle *filehandle,
             return false;
         } else if (bytes_read == 0) {
             dcm_error_set(error, DCM_ERROR_CODE_IO,
-                "End of filehandle",
-                "Needed %zd bytes beyond end of filehandle", length);
+                "end of filehandle",
+                "needed %zd bytes beyond end of filehandle", length);
             return false;
         }
 
@@ -295,8 +302,8 @@ static bool get_num_frames(DcmError **error,
     uint32_t num_frames = strtol(value, NULL, 10);
     if (num_frames == 0) {
         dcm_error_set(error, DCM_ERROR_CODE_PARSE,
-                      "Basic Offset Table read failed",
-                      "Value of Data Element 'Number of Frames' is malformed");
+                      "BasicOffsetTable read failed",
+                      "value of NumberOfFrames is malformed");
         return false;
     }
 
@@ -306,6 +313,26 @@ static bool get_num_frames(DcmError **error,
 }
 
 
+static bool get_frame_offset(DcmError **error,
+                             const DcmDataSet *metadata,
+                             uint32_t *frame_offset)
+{
+    (void) error;
+
+    // optional, defaults to 0
+    int64_t value;
+    if (!get_tag_int(NULL,
+        metadata, "ConcatenationFrameOffsetNumber", &value)) {
+        value = 0;
+    }
+
+    // it's a uint32 in the DICOM file
+    *frame_offset = (uint32_t) value;
+
+    return true;
+}
+
+
 static bool get_frame_size(DcmError **error,
                            const DcmDataSet *metadata,
                            uint32_t *frame_width,
@@ -318,9 +345,15 @@ static bool get_frame_size(DcmError **error,
         !get_tag_int(error, metadata, "Rows", &height)) {
         return false;
     }
+    if (width <= 0 || height <= 0) {
+        dcm_error_set(error, DCM_ERROR_CODE_PARSE,
+                      "frame read failed",
+                      "value of Columns or Rows is out of range");
+        return false;
+    }
 
-    *frame_width = width;
-    *frame_height = height;
+    *frame_width = (uint32_t) width;
+    *frame_height = (uint32_t) height;
 
     return true;
 }
@@ -350,8 +383,15 @@ static bool get_tiles(DcmError **error,
     height = frame_width;
     (void) get_tag_int(NULL, metadata, "TotalPixelMatrixRows", &height);
 
-    *tiles_across = width / frame_width + !!(width % frame_width);
-    *tiles_down = height / frame_height + !!(height % frame_height);
+    if (width <= 0 || height <= 0) {
+        dcm_error_set(error, DCM_ERROR_CODE_PARSE,
+                      "frame read failed",
+                      "value of TotalPixelMatrixColumns or TotalPixelMatrixRows is out of range");
+        return false;
+    }
+
+    *tiles_across = (uint32_t) width / frame_width + !!(width % frame_width);
+    *tiles_down = (uint32_t) height / frame_height + !!(height % frame_height);
 
     return true;
 }
@@ -496,8 +536,8 @@ static bool parse_preamble(DcmError **error,
 
     if (strcmp(prefix, "DICM") != 0) {
         dcm_error_set(error, DCM_ERROR_CODE_PARSE,
-                      "Reading of File Meta Information failed",
-                      "Prefix 'DICM' not found.");
+                      "reading of file meta information failed",
+                      "prefix DICM not found");
         return false;
     }
 
@@ -536,7 +576,7 @@ static DcmDataSet *dcm_filehandle_read_file_meta(DcmError **error,
                          false,
                          &parse,
                          filehandle)) {
-        return false;
+        return NULL;
     }
 
     /* Sanity check. We should have parsed a single dataset into the sequence
@@ -555,8 +595,8 @@ static DcmDataSet *dcm_filehandle_read_file_meta(DcmError **error,
     }
 
     DcmDataSet *file_meta = dcm_sequence_get(error, sequence, 0);
-    if (file_meta == NULL ) {
-        return false;
+    if (file_meta == NULL) {
+        return NULL;
     }
 
     // steal file_meta to stop it being destroyed
@@ -567,6 +607,29 @@ static DcmDataSet *dcm_filehandle_read_file_meta(DcmError **error,
 }
 
 
+static bool dcm_filehandle_set_transfer_syntax(DcmError **error,
+                                               DcmFilehandle *filehandle,
+                                               const char *transfer_syntax_uid)
+{
+    if (filehandle->transfer_syntax_uid) {
+        free(filehandle->transfer_syntax_uid);
+    }
+
+    filehandle->transfer_syntax_uid = dcm_strdup(error, transfer_syntax_uid);
+    if (filehandle->transfer_syntax_uid == NULL) {
+        return false;
+    }
+
+    if (strcmp(filehandle->transfer_syntax_uid, "1.2.840.10008.1.2") == 0) {
+        filehandle->implicit = true;
+    }
+
+    filehandle->desc.transfer_syntax_uid = filehandle->transfer_syntax_uid;
+
+    return true;
+}
+
+
 const DcmDataSet *dcm_filehandle_get_file_meta(DcmError **error,
                                                DcmFilehandle *filehandle)
 {
@@ -598,19 +661,13 @@ const DcmDataSet *dcm_filehandle_get_file_meta(DcmError **error,
             return NULL;
         }
 
-        filehandle->transfer_syntax_uid =
-            dcm_strdup(error, transfer_syntax_uid);
-        if (filehandle->transfer_syntax_uid == NULL) {
+        if (!dcm_filehandle_set_transfer_syntax(error,
+                                                filehandle,
+                                                transfer_syntax_uid)) {
             dcm_dataset_destroy(file_meta);
             return NULL;
         }
 
-        if (strcmp(filehandle->transfer_syntax_uid, "1.2.840.10008.1.2") == 0) {
-            filehandle->implicit = true;
-        }
-
-        filehandle->desc.transfer_syntax_uid = filehandle->transfer_syntax_uid;
-
         filehandle->file_meta = file_meta;
     } else {
         // move the read point to the start of the slide metadata
@@ -664,49 +721,53 @@ static bool set_pixel_description(DcmError **error,
         !dcm_element_get_value_integer(error, element, 0, &value)) {
         return false;
     }
-    desc->rows = value;
+    desc->rows = (uint16_t) value;
 
     element = dcm_dataset_get(error, metadata, 0x00280011);
     if (element == NULL ||
         !dcm_element_get_value_integer(error, element, 0, &value)) {
         return false;
     }
-    desc->columns = value;
+    desc->columns = (uint16_t) value;
 
     element = dcm_dataset_get(error, metadata, 0x00280002);
     if (element == NULL ||
         !dcm_element_get_value_integer(error, element, 0, &value)) {
         return false;
     }
-    desc->samples_per_pixel = value;
+    desc->samples_per_pixel = (uint16_t) value;
 
     element = dcm_dataset_get(error, metadata, 0x00280100);
     if (element == NULL ||
         !dcm_element_get_value_integer(error, element, 0, &value)) {
         return false;
     }
-    desc->bits_allocated = value;
+    desc->bits_allocated = (uint16_t) value;
 
     element = dcm_dataset_get(error, metadata, 0x00280101);
     if (element == NULL ||
         !dcm_element_get_value_integer(error, element, 0, &value)) {
         return false;
     }
-    desc->bits_stored = value;
+    desc->bits_stored = (uint16_t) value;
 
     element = dcm_dataset_get(error, metadata, 0x00280103);
     if (element == NULL ||
         !dcm_element_get_value_integer(error, element, 0, &value)) {
         return false;
     }
-    desc->pixel_representation = value;
+    desc->pixel_representation = (uint16_t) value;
 
-    element = dcm_dataset_get(error, metadata, 0x00280006);
-    if (element == NULL ||
-        !dcm_element_get_value_integer(error, element, 0, &value)) {
-        return false;
+    // required if samples per pixel > 1, defaults to 0 (interleaved)
+    desc->planar_configuration = 0;
+    if (desc->samples_per_pixel > 1) {
+        element = dcm_dataset_get(error, metadata, 0x00280006);
+        if (element == NULL ||
+            !dcm_element_get_value_integer(error, element, 0, &value)) {
+            return false;
+        }
+        desc->planar_configuration = (uint16_t) value;
     }
-    desc->planar_configuration = value;
 
     element = dcm_dataset_get(error, metadata, 0x00280004);
     if (element == NULL ||
@@ -765,7 +826,7 @@ DcmDataSet *dcm_filehandle_read_metadata(DcmError **error,
                            filehandle->implicit,
                            &parse,
                            filehandle)) {
-        return false;
+        return NULL;
     }
 
     /* Sanity check. We should have parsed a single dataset into the sequence
@@ -784,8 +845,8 @@ DcmDataSet *dcm_filehandle_read_metadata(DcmError **error,
     }
 
     DcmDataSet *meta = dcm_sequence_get(error, sequence, 0);
-    if (meta == NULL ) {
-        return false;
+    if (meta == NULL) {
+        return NULL;
     }
 
     // steal meta to stop it being destroyed
@@ -804,6 +865,7 @@ const DcmDataSet *dcm_filehandle_get_metadata_subset(DcmError **error,
     static uint32_t stop_tags[] = {
         TAG_REFERENCED_IMAGE_NAVIGATION_SEQUENCE,
         TAG_PER_FRAME_FUNCTIONAL_GROUP_SEQUENCE,
+        TAG_EXTENDED_OFFSET_TABLE,
         TAG_PIXEL_DATA,
         TAG_FLOAT_PIXEL_DATA,
         TAG_DOUBLE_PIXEL_DATA,
@@ -821,6 +883,9 @@ const DcmDataSet *dcm_filehandle_get_metadata_subset(DcmError **error,
         DcmDataSet *meta = dcm_filehandle_read_metadata(error,
                                                         filehandle,
                                                         stop_tags);
+        if (meta == NULL) {
+            return NULL;
+        }
 
         // record the position of the tag that stopped the read
         if (!dcm_offset(error, filehandle, &filehandle->after_read_metadata)) {
@@ -834,11 +899,12 @@ const DcmDataSet *dcm_filehandle_get_metadata_subset(DcmError **error,
                            &filehandle->frame_width,
                            &filehandle->frame_height) ||
             !get_num_frames(error, meta, &filehandle->num_frames) ||
+            !get_frame_offset(error, meta, &filehandle->frame_offset) ||
             !get_tiles(error, meta,
                 &filehandle->tiles_across, &filehandle->tiles_down) ||
             !set_pixel_description(error, meta, &filehandle->desc)) {
             dcm_dataset_destroy(meta);
-            return false;
+            return NULL;
         }
         filehandle->num_tiles = filehandle->tiles_across *
             filehandle->tiles_down;
@@ -912,8 +978,8 @@ static bool parse_frame_index_sequence_end(DcmError **error,
         if ((filehandle->column_position - 1) % filehandle->frame_width != 0 ||
             (filehandle->row_position - 1) % filehandle->frame_height != 0) {
             dcm_error_set(error, DCM_ERROR_CODE_PARSE,
-                          "Reading PerFrameFunctionalGroupsSequence failed",
-                          "Unsupported frame alignment.");
+                          "reading PerFrameFunctionalGroupsSequence failed",
+                          "unsupported frame alignment");
             return false;
         }
 
@@ -1000,6 +1066,8 @@ static bool read_frame_index(DcmError **error,
         .stop = parse_frame_index_stop,
     };
 
+    dcm_log_debug("reading PerFrameFunctionalGroupSequence");
+
     filehandle->frame_index = DCM_NEW_ARRAY(error,
                                             filehandle->num_tiles,
                                             uint32_t);
@@ -1026,10 +1094,10 @@ static bool read_frame_index(DcmError **error,
 }
 
 
-static bool parse_skip_to_index(void *client,
-                                uint32_t tag,
-                                DcmVR vr,
-                                uint32_t length)
+static bool parse_skip_to(void *client,
+                          uint32_t tag,
+                          DcmVR vr,
+                          uint32_t length)
 {
     DcmFilehandle *filehandle = (DcmFilehandle *) client;
 
@@ -1038,19 +1106,86 @@ static bool parse_skip_to_index(void *client,
 
     filehandle->last_tag = tag;
 
-    return tag == TAG_PER_FRAME_FUNCTIONAL_GROUP_SEQUENCE ||
-           tag == TAG_PIXEL_DATA ||
-           tag == TAG_FLOAT_PIXEL_DATA ||
-           tag == TAG_DOUBLE_PIXEL_DATA;
+    for (int i = 0; filehandle->skip_to_tags[i]; i++)
+        if (tag == filehandle->skip_to_tags[i])
+            return true;
+
+    return false;
+
 }
 
 
-static bool read_skip_to_index(DcmError **error,
-                             DcmFilehandle *filehandle)
+static bool read_skip_to(DcmError **error,
+                         DcmFilehandle *filehandle,
+                         uint32_t *skip_to_tags)
 
 {
     static DcmParse parse = {
-        .stop = parse_skip_to_index,
+        .stop = parse_skip_to,
+    };
+
+    filehandle->skip_to_tags = skip_to_tags;
+    if (!dcm_parse_dataset(error,
+                           filehandle->io,
+                           filehandle->implicit,
+                           &parse,
+                           filehandle)) {
+        return false;
+    }
+
+    return true;
+}
+
+
+static bool parse_extended_offsets_element_create(DcmError **error,
+                                                  void *client,
+                                                  uint32_t tag,
+                                                  DcmVR vr,
+                                                  char *value,
+                                                  uint32_t length)
+{
+    USED(error);
+    USED(vr);
+
+    DcmFilehandle *filehandle = (DcmFilehandle *) client;
+    int64_t expected_size = filehandle->num_frames * sizeof(int64_t);
+
+    if (tag == TAG_EXTENDED_OFFSET_TABLE && length == expected_size) {
+        memcpy(filehandle->offset_table, value, length);
+        filehandle->have_extended_offset_table = true;
+
+        // the size of the pixeldata header, plus the size of the empty frame
+        // 0 (the BOT)
+        filehandle->first_frame_offset = 20;
+    }
+
+    return true;
+}
+
+
+static bool parse_extended_offsets_stop(void *client,
+                                        uint32_t tag,
+                                        DcmVR vr,
+                                        uint32_t length)
+{
+    DcmFilehandle *filehandle = (DcmFilehandle *) client;
+
+    USED(vr);
+    USED(length);
+
+    filehandle->last_tag = tag;
+
+    return tag != TAG_EXTENDED_OFFSET_TABLE;
+}
+
+
+static bool
+read_extended_offsets(DcmError **error,
+                      DcmFilehandle *filehandle)
+{
+    static DcmParse parse = {
+        .element_create = parse_extended_offsets_element_create,
+        .stop = parse_extended_offsets_stop,
     };
 
     if (!dcm_parse_dataset(error,
@@ -1074,41 +1209,6 @@ bool dcm_filehandle_prepare_read_frame(DcmError **error,
             return false;
         }
 
-        if (filehandle->layout == DCM_LAYOUT_UNKNOWN) {
-            dcm_error_set(error, DCM_ERROR_CODE_PARSE,
-                          "Reading PixelData failed",
-                          "Unsupported DimensionOrganisationType.");
-            return false;
-        }
-
-        // we may have previously stopped for many reasons ... skip ahead to per
-        // frame functional group, or pixel data
-        if (!read_skip_to_index(error, filehandle)) {
-            return false;
-        }
-
-        // if we're on per frame func, read that in
-        if (filehandle->last_tag == TAG_PER_FRAME_FUNCTIONAL_GROUP_SEQUENCE &&
-            !read_frame_index(error, filehandle)) {
-            return false;
-        }
-
-        // and we must now be on pixel data
-        if (filehandle->last_tag != TAG_PIXEL_DATA &&
-            filehandle->last_tag != TAG_FLOAT_PIXEL_DATA &&
-            filehandle->last_tag != TAG_DOUBLE_PIXEL_DATA) {
-            dcm_error_set(error, DCM_ERROR_CODE_PARSE,
-                          "Reading PixelData failed",
-                          "Could not determine offset of Pixel Data Element.");
-            return false;
-        }
-
-        if (!dcm_offset(error, filehandle, &filehandle->pixel_data_offset)) {
-            return false;
-        }
-
-        // read the BOT, or build it
-        dcm_log_debug("Reading PixelData.");
         filehandle->offset_table = DCM_NEW_ARRAY(error,
                                                  filehandle->num_frames,
                                                  int64_t);
@@ -1116,28 +1216,95 @@ bool dcm_filehandle_prepare_read_frame(DcmError **error,
             return false;
         }
 
-        const char *syntax = dcm_filehandle_get_transfer_syntax_uid(filehandle);
-        if (dcm_is_encapsulated_transfer_syntax(syntax)) {
-            // read the bot if available, otherwise parse pixeldata to find
-            // offsets
-            if (!dcm_parse_pixeldata_offsets(error,
-                                             filehandle->io,
-                                             filehandle->implicit,
-                                             &filehandle->first_frame_offset,
-                                             filehandle->offset_table,
-                                             filehandle->num_frames)) {
-                return false;
-            }
-        } else {
-            for (uint32_t i = 0; i < filehandle->num_frames; i++) {
-                filehandle->offset_table[i] = i *
-                                              filehandle->desc.rows *
-                                              filehandle->desc.columns *
-                                              filehandle->desc.samples_per_pixel;
-            }
+        if (filehandle->layout == DCM_LAYOUT_UNKNOWN) {
+            dcm_error_set(error, DCM_ERROR_CODE_PARSE,
+                          "reading PixelData failed",
+                          "unsupported DimensionOrganisationType");
+            return false;
+        }
 
-            // Header of Pixel Data Element
-            filehandle->first_frame_offset = 12;
+        // skip ahead to per frame functional group, if present, and read it
+        uint32_t skip_to_per_frame[] = {
+            TAG_PER_FRAME_FUNCTIONAL_GROUP_SEQUENCE,
+            TAG_EXTENDED_OFFSET_TABLE,
+            TAG_PIXEL_DATA,
+            TAG_FLOAT_PIXEL_DATA,
+            TAG_DOUBLE_PIXEL_DATA,
+            0
+        };
+        if (!read_skip_to(error, filehandle, skip_to_per_frame)) {
+            return false;
+        }
+        if (filehandle->last_tag == TAG_PER_FRAME_FUNCTIONAL_GROUP_SEQUENCE &&
+            !read_frame_index(error, filehandle)) {
+            return false;
+        }
+
+        // skip ahead to extended offset table, if present
+        uint32_t skip_to_offset[] = {
+            TAG_EXTENDED_OFFSET_TABLE,
+            TAG_PIXEL_DATA,
+            TAG_FLOAT_PIXEL_DATA,
+            TAG_DOUBLE_PIXEL_DATA,
+            0
+        };
+        if (!read_skip_to(error, filehandle, skip_to_offset)) {
+            return false;
+        }
+        if (filehandle->last_tag == TAG_EXTENDED_OFFSET_TABLE &&
+            !read_extended_offsets(error, filehandle)) {
+            return false;
+        }
+
+        // skip to pixel data
+        uint32_t skip_to_pixel_data[] = {
+            TAG_PIXEL_DATA,
+            TAG_FLOAT_PIXEL_DATA,
+            TAG_DOUBLE_PIXEL_DATA,
+            0
+        };
+        if (!read_skip_to(error, filehandle, skip_to_pixel_data)) {
+            return false;
+        }
+        if (filehandle->last_tag != TAG_PIXEL_DATA &&
+            filehandle->last_tag != TAG_FLOAT_PIXEL_DATA &&
+            filehandle->last_tag != TAG_DOUBLE_PIXEL_DATA) {
+            dcm_error_set(error, DCM_ERROR_CODE_PARSE,
+                          "reading PixelData failed",
+                          "could not determine offset of PixelData element");
+            return false;
+        }
+
+        if (!dcm_offset(error, filehandle, &filehandle->pixel_data_offset)) {
+            return false;
+        }
+
+        // if there was no ext offset table, we must read the basic one, or
+        // create it
+        if (!filehandle->have_extended_offset_table) {
+            const char *syntax =
+                dcm_filehandle_get_transfer_syntax_uid(filehandle);
+            if (dcm_is_encapsulated_transfer_syntax(syntax)) {
+                // read the bot if available, otherwise parse pixeldata to find
+                // offsets
+                if (!dcm_parse_pixeldata_offsets(error,
+                                                 filehandle->io,
+                                                 filehandle->implicit,
+                                                 &filehandle->first_frame_offset,
+                                                 filehandle->offset_table,
+                                                 filehandle->num_frames)) {
+                    return false;
+                }
+            } else {
+                for (uint32_t i = 0; i < filehandle->num_frames; i++) {
+                    filehandle->offset_table[i] = i *
+                                                  filehandle->desc.rows *
+                                                  filehandle->desc.columns *
+                                                  filehandle->desc.samples_per_pixel;
+                }
+
+                filehandle->first_frame_offset = 12;
+            }
         }
     } else {
         // always position at pixel_data
@@ -1154,7 +1321,7 @@ DcmFrame *dcm_filehandle_read_frame(DcmError **error,
                                     DcmFilehandle *filehandle,
                                     uint32_t frame_number)
 {
-    dcm_log_debug("Read frame number #%u.", frame_number);
+    dcm_log_debug("read frame number #%u", frame_number);
 
     if (!dcm_filehandle_prepare_read_frame(error, filehandle)) {
         return NULL;
@@ -1162,14 +1329,14 @@ DcmFrame *dcm_filehandle_read_frame(DcmError **error,
 
     if (frame_number == 0) {
         dcm_error_set(error, DCM_ERROR_CODE_PARSE,
-                      "Reading Frame Item failed",
-                      "Frame Number must be non-zero");
+                      "reading frame item failed",
+                      "frame number must be non-zero");
         return NULL;
     }
     if (frame_number > filehandle->num_frames) {
         dcm_error_set(error, DCM_ERROR_CODE_PARSE,
-                      "Reading Frame Item failed",
-                      "Frame Number must be less than %u",
+                      "reading frame item failed",
+                      "frame number must be less than %u",
                       filehandle->num_frames);
         return NULL;
     }
@@ -1210,6 +1377,56 @@ DcmFrame *dcm_filehandle_read_frame(DcmError **error,
 }
 
 
+bool dcm_filehandle_get_frame_number(DcmError **error,
+                                     DcmFilehandle *filehandle,
+                                     uint32_t column,
+                                     uint32_t row,
+                                     uint32_t *frame_number)
+{
+    dcm_log_debug("Get frame number at (%u, %u)", column, row);
+
+    if (!dcm_filehandle_prepare_read_frame(error, filehandle)) {
+        return false;
+    }
+
+    if (column >= filehandle->tiles_across ||
+        row >= filehandle->tiles_down) {
+        dcm_error_set(error, DCM_ERROR_CODE_PARSE,
+                      "reading Frame position failed",
+                      "column and Row must be less than %u, %u",
+                      filehandle->tiles_across,
+                      filehandle->tiles_down);
+        return false;
+    }
+
+    int64_t index = column + row * filehandle->tiles_across;
+    if (filehandle->layout == DCM_LAYOUT_SPARSE) {
+        index = filehandle->frame_index[index];
+        if (index == 0xffffffff) {
+            dcm_error_set(error, DCM_ERROR_CODE_MISSING_FRAME,
+                          "no frame",
+                          "no frame at position (%u, %u)", column, row);
+            return false;
+        }
+    } else {
+        // subtract the start of this file, for catenation support
+        index -= filehandle->frame_offset;
+        if (index < 0 || index >= (int64_t) filehandle->num_frames) {
+            dcm_error_set(error, DCM_ERROR_CODE_MISSING_FRAME,
+                          "no frame",
+                          "no frame at position (%u, %u)", column, row);
+            return false;
+        }
+    }
+
+    // frame numbers are from 1, and are always uint32
+    if (frame_number)
+        *frame_number = (uint32_t) (index + 1);
+
+    return true;
+}
+
+
 DcmFrame *dcm_filehandle_read_frame_position(DcmError **error,
                                              DcmFilehandle *filehandle,
                                              uint32_t column,
@@ -1217,33 +1434,13 @@ DcmFrame *dcm_filehandle_read_frame_position(DcmError **error,
 {
     dcm_log_debug("Read frame position (%u, %u)", column, row);
 
-    if (!dcm_filehandle_prepare_read_frame(error, filehandle)) {
+    uint32_t frame_number;
+    if (!dcm_filehandle_get_frame_number(error,
+            filehandle, column, row, &frame_number)) {
         return NULL;
     }
 
-    if (column >= filehandle->tiles_across ||
-        row >= filehandle->tiles_down) {
-        dcm_error_set(error, DCM_ERROR_CODE_PARSE,
-                      "Reading Frame position failed",
-                      "Column and row must be less than %u, %u",
-                      filehandle->tiles_across,
-                      filehandle->tiles_down);
-        return NULL;
-    }
-
-    uint32_t index = column + row * filehandle->tiles_across;
-    if (filehandle->layout == DCM_LAYOUT_SPARSE) {
-        index = filehandle->frame_index[index];
-        if (index == 0xffffffff) {
-            dcm_error_set(error, DCM_ERROR_CODE_MISSING_FRAME,
-                          "No frame",
-                          "No Frame at position (%u, %u)", column, row);
-            return NULL;
-        }
-    }
-
-    // read_frame() numbers from 1
-    return dcm_filehandle_read_frame(error, filehandle, index + 1);
+    return dcm_filehandle_read_frame(error, filehandle, frame_number);
 }
 
 
@@ -1387,7 +1584,15 @@ static bool print_element_create(DcmError **error,
 {
     DcmFilehandle *filehandle = (DcmFilehandle *) client;
 
-    USED(error);
+    // we must record the transfer syntax in the file meta since we need it
+    // to determine implicit/explicit encoding
+    if (tag == TAG_TRANSFER_SYNTAX_UID &&
+        vr == DCM_VR_UI &&
+        value &&
+        length > 10 &&
+        !dcm_filehandle_set_transfer_syntax(error, filehandle, value)) {
+        return false;
+    }
 
     printf("%*.*s(%04x,%04x) ",
            filehandle->indent * 2,
@@ -1443,9 +1648,10 @@ static bool print_pixeldata_create(DcmError **error,
 
     uint32_t n = MIN(16, length);
     for (uint32_t i = 0; i < n; i++) {
-        printf("%02x", value[i]);
+        printf("%02x", value[i] & 0xff);
 
-        if (i % size == size - 1) {
+        if (size > 0 &&
+            i % size == size - 1) {
             printf(" ");
         }
     }
@@ -1499,8 +1705,7 @@ bool dcm_filehandle_print(DcmError **error,
     dcm_log_info("Read metadata");
     if (!dcm_parse_dataset(error,
                            filehandle->io,
-                           // FIXME we should check transfer syntax for this
-                           false,
+                           filehandle->implicit,
                            &parse,
                            filehandle)) {
         return false;
